-module(customer).
-include("customer.hrl").
-behaviour(gen_server).
        
%% Accessors
get_value() ->
    gen_server:call(?MODULE, get_value).

get_position() ->
    gen_server:call(?MODULE, get_position).

set_value(V) ->
    gen_server:cast(?MODULE, {set_value, V}).

set_position(P) ->
    gen_server:cast(?MODULE, {set_position, P}).

remove_value(V) ->
    gen_server:cast(?MODULE, {remove_value, V}).

add_value(V) ->
    gen_server:cast(?MODULE, {remove_value, -V}).

%% API
%% 

start_link() ->
    {ok, Lat} = application:get_env(?MODULE,lat), 
    {ok, Lng} = application:get_env(?MODULE,lng), 
    {ok, Val} = application:get_env(?MODULE,val),
    start_link({Lat,Lng}, Val).


start_link(P, V) ->
    State = #state{position = P, value = V},
    gen_server:start_link({local, ?MODULE}, ?MODULE, State, []).


login() ->
    %% the login name is the node name
    gen_server:call(?MODULE, login).

logout() ->
    gen_server:cast(?MODULE, logout).

recycle() ->
    gen_server:cast(?MODULE, recycle).

recycle(V) ->
    gen_server:cast(?MODULE, {recycle, V}).

stop() ->
    gen_server:cast(?MODULE, stop).

%% Initialization
init(State) ->
    {ok, State}.


%% Calls
handle_call(get_value, _From, State = #state{value=V}) ->
    {reply, V, State};
handle_call(get_position, _From, State = #state{position=P}) ->
    {reply, P, State};
handle_call(login, _From, State = #state{position=P,value=V}) ->
    Reply = df:subscribe(?MODULE,P,V),
    {reply, Reply, State}.


%% Casts
handle_cast({set_value, V}, State = #state{position=P}) ->
    df:update(?MODULE, P, V),
    {noreply, State#state{value=V}};

handle_cast({set_position, P}, State = #state{value=V}) ->
    df:update(?MODULE, P, V),
    {noreply, State#state{position=P}};

handle_cast({remove_value, RemoveV}, State = #state{position=P, value = V}) ->
    NewValue = V - RemoveV,
    df:update(?MODULE, P, NewValue),
    {noreply, State#state{value = NewValue}};

handle_cast(recycle, State = #state{value = V}) when V =< 0 ->
    {stop, wrong_value, State};

handle_cast(recycle, State = #state{position = P, value = V}) ->
    customer_initiator:start({P,V}),
    {noreply, State};

handle_cast({recycle, V}, State) when V =< 0 ->
    {stop, wrong_value, State};

handle_cast({recycle, V}, State = #state{position = P}) ->
    customer_initiator:start({P,V}),
    {noreply, State};


handle_cast(logout, State) ->
    df:unsubscribe(),
    {noreply, State};

handle_cast(stop, _State) ->
    {stop, normal, _State}.


%% Cleanup
terminate(_Reason, _) ->
    logout(),
    ok.


