-module(customer_web).
-behaviour(gen_server).

%% Wrap a misultin server around a gen_server behaviour.

start_link(Port) ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [Port], []).

init([Port]) ->
    misultin:start_link([{port, Port}, {loop, fun(Req) -> handle_http(Req) end}]). % and then start the web server

%% Cleanup
terminate(normal, _) ->
    misultin:stop();

terminate(shutdown, _) ->
    misultin:stop().

%% Web Server

www_dir() ->
    code:priv_dir(customer) ++ "/www".

handle_http(Req) ->
    % dispatch
    handle(Req:get(method), Req:resource([lowercase, urldecode]), Req).

handle('GET', [], Req) ->
    case whereis(customer) of
        undefined -> 
            Req:file(www_dir() ++ "/login.html");
        _ ->
            Req:file(www_dir() ++ "/home.html")
    end;

handle('POST', [], Req) ->
    %% Get the post arguments
    Args = Req:parse_post(),
    Val = try 
              list_to_integer(proplists:get_value("val", Args))
          catch
              error:badarg ->                         % The user entered a floated quantity
                  list_to_float(proplists:get_value("val", Args))
          end,
        
    Lat = list_to_float(proplists:get_value("lat",Args)),
    Lng = list_to_float(proplists:get_value("lng",Args)),

    Pos = {Lat, Lng},

    %% Customer Server specification
    CustomerServer = { customer, 
                       {customer, start_link, [Pos, Val]}, 
                       transient,
                       10000,
                       worker,
                       [customer,customer_initiator] },

    %% Start the customer server through the supervisor
    supervisor:start_child(customer_sup, CustomerServer),
    
    %% Customer Login
    customer:login(),

    %% Go to the home screen
    Req:file(www_dir() ++ "/home.html");

handle('GET', ["ajax"], Req) ->
    %% json response
    Data = jsonerl:encode(df:list_all()),
    Req:ok(Data);

handle('GET', ["add"], Req) ->
    Args = Req:parse_qs(),
    Val = try 
              list_to_integer(proplists:get_value("val", Args))
          catch
              error:badarg ->                         % The user entered a floated quantity
                  list_to_float(proplists:get_value("val", Args))
          end,
    
    %% Add the requested quantity
    customer:add_value(Val),
    Req:ok("");                                 % empty response

handle('GET', ["recycle"], Req) ->
    %% Perform recycle
    customer:recycle(),
    Req:ok("");

handle('GET', ["img", Image], Req) ->
    %% 
    Req:file(www_dir() ++ "/img/" ++ Image);

handle(_,_,Req) ->
    Req:ok("Page not found.").
