-module(df).
-behaviour(gen_server).

%% API
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []). % start , register, call init with empty arguments

subscribe(Type, Pos, Val) when Type == truck ; Type == customer ->
    gen_server:call({?MODULE, ?SERVER}, {subscribe, self(), node(), Type, Pos, Val}).

update(Type, Pos, Val) when Type == truck ; Type == customer ->
    gen_server:cast({?MODULE, ?SERVER}, {update, self(), node(), Type, Pos, Val}).

unsubscribe() ->
    gen_server:cast({?MODULE, ?SERVER}, {unsubscribe, self()}).

list_trucks() ->
    gen_server:call({?MODULE, ?SERVER}, list_trucks).

list_all() ->
    gen_server:call({?MODULE, ?SERVER}, list_all).

stop() ->
    gen_server:cast(?MODULE, stop).

%% Initialization
init(_Args) ->
    process_flag(trap_exit, true),
    df_db:create_table(),
    {ok, []}.

%% Calls
handle_call({subscribe, Pid, Node, Type, {Lat, Lng}, Val}, _From, State) ->
    Reply = case df_db:member(Pid) of
                true ->
                    {error, already_subscribed};
                false ->
                    erlang:monitor(process,Pid),
                    df_db:insert({Pid,Node,Type, Lat, Lng, Val}),
                    {ok, subscribed}
            end,
    {reply, Reply, State};


handle_call(list_trucks, _From, State) ->
    {reply, df_db:list_trucks(), State};

handle_call(list_all, _From, State) ->
    {reply, df_db:list_all(), State}.


%% Casts
handle_cast({update, Pid, Node, Type, {Lat, Lng}, Val}, State) ->
    case df_db:member(Pid) of
        true ->
            df_db:insert({Pid,Node,Type, Lat, Lng, Val});
        false ->
            ok
    end,
    {noreply, State};

handle_cast({unsubscribe, Pid}, State) ->
    df_db:delete(Pid),
    {noreply, State};

handle_cast(stop, State) ->
    {stop, normal, State}.


%% Other msgs
handle_info({'DOWN', _Ref, process, Pid, _Reason}, State) ->
    df_db:delete(Pid),
    {noreply, State}.


%% Cleanup
terminate(normal, _) ->
    %% Normal stop, the exit does not propagate on clients.
    df_db:delete_table();

terminate(shutdown, _) ->
    %% Shutdown stop from the supervisor, the exit propagates.
    df_db:delete_table().


