-module(df_web).
-behaviour(gen_server).

%% Wrap a misultin server around a gen_server behaviour.

start_link(Port) ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [Port], []).

init([Port]) ->
    misultin:start_link([{port, Port}, {loop, fun(Req) -> handle_http(Req) end}]). % and then start the web server

%% Cleanup
terminate(normal, _) ->
    misultin:stop();

terminate(shutdown, _) ->
    misultin:stop().

%% Web Server

www_dir() ->
    code:priv_dir(df) ++ "/www".

handle_http(Req) ->
    % dispatch
    handle(Req:get(method), Req:resource([lowercase, urldecode]), Req).

handle('GET', [], Req) ->
    Req:file(www_dir() ++ "/home.html");

handle('GET', ["ajax"], Req) ->
    %% json response
    Data = jsonerl:encode(df:list_all()),
    Req:ok(Data);

handle('GET', ["img", Image], Req) ->
    %% 
    Req:file(www_dir() ++ "/img/" ++ Image);

handle(_,_,Req) ->
    Req:ok("Page not found.").
