-module(truck_driver).
-include("truck.hrl").
-behaviour(gen_server).

%% API
%% 
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

run_job(Job) ->
    gen_server:cast(?MODULE, {run_job, Job}).


%% Initialization
%% 

init(_Args) ->
    {ok, []}.


%% Casts
%% 
handle_cast({run_job, Job}, State) ->
    Run_step = fun (Step = { _ , Duration}) ->
                       timer:sleep(round(timer:seconds(Duration) * ?PROG_SPEED)),
                       truck:step_done(Step)
               end,

    DriveSteps = gmaps:compute_steps(Job#job.start_position,Job#job.end_position),

    %% Create the pickup step
    %% 
    PickupTime = abs(gmaps:compute_pickup(Job#job.value)),
    PickupStep = {Job#job.end_position, PickupTime},

    %% Append it to the drive steps
    Steps = lists:append(DriveSteps, [PickupStep]),

    %% Stepping
    lists:foreach(Run_step, Steps),


    %% Job Done
    TimeElapsed = Job#job.eta,                  % includes pickup time
    truck:job_done(TimeElapsed),
    {noreply, State}.
