-module(truck_responder).
-behaviour(gen_fsm).

%% Callbacks
%% 

start() ->
    %% no registration
    gen_fsm:start_link(?MODULE, [], []).

init(_Args) ->
    {ok, handle_cfp, []}.

terminate(normal, _StateName, _StateData) ->    
    ok.


%% Interface

pass_cfp(Responder,Cfp) ->
    gen_fsm:send_event(Responder, Cfp).

update(Responder, Pos, Val) ->
    gen_fsm:send_event(Responder, {update, Pos, Val}).


%% States
%% 

%% Handle Cfp
handle_cfp({cfp, Initiator, Contract = {P, V}}, State) -> 
    %% process the CFP
    {FPosition, FCapacity, FTime} = truck:get_future_state(),
    case FCapacity >= V of
        true ->
            %% Enough capacity
            JobTime = gmaps:compute_eta(FPosition, P) + gmaps:compute_pickup(V),
            Eta = FTime + JobTime,
            customer_initiator:propose(Initiator, Eta),
            %% add the initiator pid to the state and the FTime to avoid race condition
            {next_state, handle_acceptance, {Initiator, Contract, FTime, JobTime}};

        false ->
            %% Low capacity - Refuse
            customer_initiator:refuse(Initiator, exceeds_capacity),
            {stop, normal, State}
    end.

%% Handle Reject
handle_acceptance(reject_proposal, State) ->
    {stop, normal, State};

%% Handle Accept
handle_acceptance(accept_proposal, State = {Initiator, Contract, FTime, JobTime}) ->
    case truck:add_goal(self(), Contract, FTime, JobTime) of
        {ok, goal_added} ->
            {next_state, handle_done, State};
        {error, race_condition} ->
            gen_fsm:send_event(Initiator, {failure, race_condition}),
            {stop, normal, State}
    end.


%% Handle Done
handle_done({done, TimeElapsed}, State = {Initiator, _Contract, _FTime, _JobTime}) ->
    unlink(Initiator),
    gen_fsm:send_event(Initiator, {inform, TimeElapsed}),
    {stop, normal, State};

handle_done({update, Pos, Val}, State = {Initiator, _Contract, _FTime, _JobTime}) ->
    customer_initiator:update(Initiator, Pos, Val),
    {next_state, handle_done, State}.




